;
; ***************************************************************************
; 90 asteinen CCPM mikseri
;
; HW: PIC16F84
;     12MHz xtal
; ***************************************************************************
;

        LIST    p=16F84 ; PIC16F844 is the target processor

	#include "P16F84.INC" ; Include header file

	__CONFIG _WDT_OFF & _HS_OSC & _PWRTE_ON
        
	cblock 0x0c
	  state
	  pin
	  hiB   	; MSB of measurement.
	  lowB 		; LSB of measurement.
	  hiInA		
	  loInA
	  hiInE		
	  loInE
	  hiInP		
	  loInP
	  hiOutR		
	  loOutR
	  hiOutC		
	  loOutC
	  hiOutL		
	  loOutL
	endc

; Device data and reset vector
        org 0
	goto start

; Table to convert pin number (0-7) into bit mask (00000001b to 10000000b).

Pinz         ADDWF PCL,F                  
             RETLW d'1'                 
             RETLW d'2'
             RETLW d'4'
             RETLW d'8'
             RETLW d'16'
             RETLW d'32'
             RETLW d'64'
             RETLW d'128'

;********************************************************************
;               
;*********************************************************************
start   
	bsf	STATUS,RP0	; page 1
	movlw	B'00111111'
;	  	  |||||||'--- \
;                 ||||||'----  | prescaler ratio 1/256
;		  |||||'----- /
;		  ||||'------ prescaler to TMR0
;		  |||'------- TMR0 clock edge select = falling
;		  ||'-------- TMR0 clock source = RA4 (external)
;		  |'--------- external int edge 
;		  '---------- internal pullups = disabled
	movwf	OPTION_REG
	bcf	TRISB,0		; outputR  (right) 
	bcf	TRISB,1		; outputC  (back/center)
	bcf	TRISB,2		; outputL  (left) 
	bsf	TRISB,3		; inputA (aileron) ch2 
	bsf	TRISB,4		; inputE (elevator) ch3 
	bsf	TRISB,5		; inputP  (pitch) ch6 
	bcf	STATUS,RP0	; page 0


mainloop

	; mitataan servopulssien pituudet ja siirretn ne 
	; samalla nollan ymprille vhentmll mittaustuloksesta
	; 1.5ms jaksoa vastaava laskurin arvo

	call 	Pulsin_IA	; aileron
	call	minus15ms
	movf 	hiB,W
	movwf	hiInA
	movf 	lowB,W
	movwf	loInA

           
	call 	Pulsin_IE	; elevator
	call	minus15ms
	movf 	hiB,W
	movwf	hiInE
	movf 	lowB,W
	movwf	loInE

	call 	Pulsin_IP	; pitch
	call	minus15ms
	movf 	hiB,W
	movwf	hiInP
	movf 	lowB,W
	movwf	loInP

	; lasketaan miksaukset

	; pitch (inP) menee positiivisena kaikille servoille, joten laitetaan se 'pohjaksi'

	movf	loInP,W
	movwf	loOutR
	movwf	loOutC
	movwf	loOutL
	movf	hiInP,W
	movwf	hiOutR
	movwf	hiOutC
	movwf	hiOutL

	; elevator (inE) menee takaservolle (outC) negatiivisena

	movf	loInE,W
	subwf	loOutC,F
	movf	hiInE,W
	btfss	STATUS,C
	incfsz	hiInE,W
	subwf	hiOutC,F

	; aileron (inA) menee vasemmalle (outL) positiivisena...

	movf	loInA,W
	addwf	loOutL,F
	movf	hiInA,W
	btfsc	STATUS,C
	incfsz	hiInA,W
	addwf	hiOutL,F

	; ja oikealle (outR) negatiivisena

	movf	loInA,W
	subwf	loOutR,F
	movf	hiInA,W
	btfss	STATUS,C
	incfsz	hiInA,W
	subwf	hiOutR,F




	; muodostetaan ulostulopulssit
	; ennen pulsin-funktion kutsumista arvoon listn viel 
	; keskiasentoa vastaavan 1.5ms antava laskurinarvo

	; varmistetaan aluksi, ett kaikki ovat nollassa, jotta 
	; saadaan positiiviset pulssit aikaiseksi

	bcf	PORTB,0
	bcf	PORTB,1
	bcf	PORTB,2

	movf	hiOutR,W
	movwf 	hiB
	movf	loOutR,W
	movwf	lowB
	call 	plus15ms
	call	Pulsout_OR	; right
	
	movf	hiOutC,W
	movwf 	hiB
	movf	loOutC,W
	movwf	lowB
	call	negateB		; takaservon pyrimissuunta vaihdetaan
	call 	plus15ms
	call	Pulsout_OC	; center/back

	movf	hiOutL,W
	movwf 	hiB
	movf	loOutL,W
	movwf	lowB
	call 	plus15ms
	call	Pulsout_OL	; left

	GOTO mainloop                 ; Do it again.





; lis hiB:loB muuttujaan 1.5ms aikaa vastaavan lukeman (643)
plus15ms
	movlw	low d'643'
	addwf	lowB,F
	movlw	high d'643'
	btfsc	STATUS,C
	movlw	(high d'643') +1
	addwf	hiB,F
	return	

; vhent hiN:loB muttujasta 1.5 aikaa vastaavan lukeman (643)
minus15ms
	movlw	low d'643'
        subwf   lowB,F
	movlw	high d'643'
	btfss	STATUS,C
	movlw	(high d'643') +1
        subwf	hiB,F           
	return

; knt luvun negatiiviseksi, eli vaihtaa pyrimissuuntaa
negateB
	comf	lowB,f           ;Complement all bytes
        comf    hiB,f

        incf    lowB,f          ;Inc. low byte always
        skpnz                   ;Skip if no carry to higher bytes
        incf    hiB,f  	       ;Carry to next byte

	return	
;***************************************************************************************
; Mittaa positiivisen pulssin pituuden pinnist PORTB,3
;
; pituuden yksikkn on 7 kskyjaksoa (xtal/4)
;
; 1 ms aikaa vastaa laskurin arvo:
;
;	xtal = 12MHz
;	1ms * ((12MHz / 4) / 7) = 429                
;***************************************************************************************             


Pulsin_IA	clrf lowB                  ; Clear the 16-bit counter.
             	clrf hiB                   

Pulsin_IA_0	btfsc PORTB,3		; varmistetaan, ett pinni on aluksi nollassa
		goto Pulsin_IA_0
            
Pulsin_IA_1				; odotellaan, jotta pinni nousee ykkseksi
             	btfsc PORTB,3
             	goto Pulsin_IA_2
             	incf lowB,F                ; Otherwise, lowB = lowB+1.
             	btfsc STATUS,Z             ; Overflow in lowB?
             	incfsz hiB,F               ; Yes: hiB=hiB+1, watch for overflow.
             	goto Pulsin_IA_1           ; Sample the pin again.
		retlw 1			; ei jaksa odottaa en

Pulsin_IA_2
		clrf lowB                  ; Clear the 16-bit counter.
             	clrf hiB
Pulsin_IA_3				; odotellaan, jotta pinni tippuu nollaksi
             	btfss PORTB,3
             	goto Pulsin_IA_4
             	incf lowB,F                ; Otherwise, lowB = lowB+1.
             	btfsc STATUS,Z             ; Overflow in lowB?
             	incfsz hiB,F               ; Yes: hiB=hiB+1, watch for overflow.
             	goto Pulsin_IA_3           ; Sample the pin again.
		retlw 2			; ei jaksa nin pitki pulsseja vahtia
Pulsin_IA_4				; loppu hyvin, kaikki hyvin.
		retlw 0

;***************************************************************************************
; Mittaa positiivisen pulssin pituuden pinnist PORTB,4
;
; pituuden yksikkn on 7 kskyjaksoa (xtal/4)
;
; 1 ms aikaa vastaa laskurin arvo:
;
;	xtal = 12MHz
;	1ms * ((12MHz / 4) / 7) = 429                
;***************************************************************************************             


Pulsin_IE	clrf lowB                  ; Clear the 16-bit counter.
             	clrf hiB                   
           
Pulsin_IE_1				; odotellaan, jotta pinni nousee ykkseksi
             	btfsc PORTB,4
             	goto Pulsin_IE_2
             	incf lowB,F                ; Otherwise, lowB = lowB+1.
             	btfsc STATUS,Z             ; Overflow in lowB?
             	incfsz hiB,F               ; Yes: hiB=hiB+1, watch for overflow.
             	goto Pulsin_IE_1           ; Sample the pin again.
		retlw 1			; ei jaksa odottaa en

Pulsin_IE_2
		clrf lowB                  ; Clear the 16-bit counter.
             	clrf hiB
Pulsin_IE_3				; odotellaan, jotta pinni tippuu nollaksi
             	btfss PORTB,4
             	goto Pulsin_IE_4
             	incf lowB,F                ; Otherwise, lowB = lowB+1.
             	btfsc STATUS,Z             ; Overflow in lowB?
             	incfsz hiB,F               ; Yes: hiB=hiB+1, watch for overflow.
             	goto Pulsin_IE_3           ; Sample the pin again.
		retlw 2			; ei jaksa nin pitki pulsseja vahtia
Pulsin_IE_4				; loppu hyvin, kaikki hyvin.
		retlw 0

;***************************************************************************************
; Mittaa positiivisen pulssin pituuden pinnist PORTB,5
;
; pituuden yksikkn on 7 kskyjaksoa (xtal/4)
;
; 1 ms aikaa vastaa laskurin arvo:
;
;	xtal = 12MHz
;	1ms * ((12MHz / 4) / 7) = 429                
;***************************************************************************************             


Pulsin_IP	clrf lowB                  ; Clear the 16-bit counter.
             	clrf hiB                   
           
Pulsin_IP_1				; odotellaan, jotta pinni nousee ykkseksi
             	btfsc PORTB,5
             	goto Pulsin_IP_2
             	incf lowB,F                ; Otherwise, lowB = lowB+1.
             	btfsc STATUS,Z             ; Overflow in lowB?
             	incfsz hiB,F               ; Yes: hiB=hiB+1, watch for overflow.
             	goto Pulsin_IP_1           ; Sample the pin again.
		retlw 1			; ei jaksa odottaa en

Pulsin_IP_2
		clrf lowB                  ; Clear the 16-bit counter.
             	clrf hiB
Pulsin_IP_3				; odotellaan, jotta pinni tippuu nollaksi
             	btfss PORTB,5
             	goto Pulsin_IP_4
             	incf lowB,F                ; Otherwise, lowB = lowB+1.
             	btfsc STATUS,Z             ; Overflow in lowB?
             	incfsz hiB,F               ; Yes: hiB=hiB+1, watch for overflow.
             	goto Pulsin_IP_3           ; Sample the pin again.
		retlw 2			; ei jaksa nin pitki pulsseja vahtia
Pulsin_IP_4				; loppu hyvin, kaikki hyvin.
		retlw 0

;
; PULSOUT port, pin, time
; Generates an output pulse in 10-cycle (10 us at 4 MHz) units,
; based on a 16-bit (1 to 65,535) value. The pulse is the reverse
; of the pin's state when pulsout is called. For instance,      if the
; specified pin is initially 1, Pulsout will invert it to make a
; negative-going pulse.

; Upon entry, the desired pin must already be set up as an output.
; The w register contains a number representing the output port (0-2) for
; RA through RC. Variable "pin" contains the pin number (0-7). The variables
; hiB and lowB are the MSB and LSB of the time argument.
Pulsout_OR
	MOVLW d'0'                 ; Pin 0.
	MOVWF pin
	MOVLW d'1'                 ; of port rb.
	CALL Pulsout               ; Pulse pin high for 100 cycles.
     	return
Pulsout_OC
	MOVLW d'1'                 ; Pin 1.
	MOVWF pin
	MOVLW d'1'                 ; of port rb.
	CALL Pulsout               ; Pulse pin high for 100 cycles.
     	return
Pulsout_OL
	MOVLW d'2'                 ; Pin 2.
	MOVWF pin
	MOVLW d'1'                 ; of port rb.
	CALL Pulsout               ; Pulse pin high for 100 cycles.
     	return

Pulsout      MOVWF FSR                  ; Point to the port number.
             MOVLW 5h                   ; Add offset for port RA.
             ADDWF FSR,F
             MOVF pin,W                 
             CALL Pinz                  ; Get bit mask from the table.
             MOVWF pin                  ; Put the mask into pin
             COMF hiB,F                 ; Take twos complement
             COMF lowB,F                ; of the 16-bit counter
             INCF lowB,F                 
             BTFSC STATUS,Z             ; If zero, lowB overflowed,
             INCF hiB,F                 ; so carry into hiB.
             MOVF pin,W                 ; Invert the selected pin.
             XORWF INDF,F

; The main timing loop. Remove the nops for 5-cycle resolution.
Pulsout_loop GOTO $+1                   ; Two-cycle "nop."
             INCF lowB,F                ; lowB = lowB+1.
             BTFSC STATUS,Z             ; Overflow in lowB?
             INCFSZ hiB,F               ; Yes: hiB=hiB+1, watch for overflow.
             GOTO Pulsout_loop          ; If not overflow, do it again.
             MOVF pin,W                 ; Invert pin (back to initial state).
             XORWF INDF,F
             RETLW 0h  



	end                 
